# Generated by Django 4.1 on 2023-10-21 09:32

import demoproject.demoapp.models
import strategy_field.fields
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="DemoAllModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("choice", strategy_field.fields.StrategyClassField()),
                ("multiple", strategy_field.fields.MultipleStrategyClassField()),
                ("custom", strategy_field.fields.StrategyField()),
                ("custom_multiple", strategy_field.fields.MultipleStrategyField()),
            ],
        ),
        migrations.CreateModel(
            name="DemoCallableModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("sender", strategy_field.fields.StrategyClassField()),
            ],
        ),
        migrations.CreateModel(
            name="DemoCustomModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("sender", strategy_field.fields.StrategyField()),
            ],
        ),
        migrations.CreateModel(
            name="DemoModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("sender", strategy_field.fields.StrategyClassField()),
            ],
        ),
        migrations.CreateModel(
            name="DemoModelCallableDefault",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "sender",
                    strategy_field.fields.StrategyClassField(
                        default=demoproject.demoapp.models.cc, null=True
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="DemoModelContext",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="DemoModelDefault",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "sender",
                    strategy_field.fields.StrategyClassField(
                        default="demoproject.demoapp.models.Sender1", null=True
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="DemoModelNone",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "sender",
                    strategy_field.fields.StrategyClassField(blank=True, null=True),
                ),
            ],
        ),
        migrations.CreateModel(
            name="DemoModelNoRegistry",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "klass",
                    strategy_field.fields.StrategyClassField(blank=True, null=True),
                ),
                (
                    "instance",
                    strategy_field.fields.StrategyField(blank=True, null=True),
                ),
            ],
        ),
        migrations.CreateModel(
            name="DemoMultipleCustomModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("sender", strategy_field.fields.MultipleStrategyField()),
            ],
        ),
        migrations.CreateModel(
            name="DemoMultipleModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "sender",
                    strategy_field.fields.MultipleStrategyClassField(
                        blank=True, null=True
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="DemoModelProxy",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("demoapp.demomodel",),
        ),
    ]
